from SciQLopPlots import QCustomPlot, QCP, QCPColorMap, QCPRange, QCPColorScale, QCPAxis, QCPColorGradient, QCPMarginGroup
from PySide6.QtWidgets import QMainWindow, QApplication
from PySide6.QtGui import QPen, QColorConstants, QColor, QBrush
import sys
import math
import numpy as np
from types import SimpleNamespace


class MainWindow(QMainWindow):
    def __init__(self):
        QMainWindow.__init__(self)
        self._setup_ui()
        self.setGeometry(400, 250, 542, 390);

    def _setup_ui(self):
        plot: QCustomPlot = QCustomPlot(self)
        plot.setInteractions(QCP.iRangeDrag|QCP.iRangeZoom|QCP.iSelectPlottables)
        self.setCentralWidget(plot)
        self.graph = plot.addSciQLopColorMap(plot.xAxis, plot.yAxis, 'MyColorMap')
        nx = 2000
        ny = 2000
        data = np.empty((nx,ny))
        x_vect= (np.arange(nx)*8./nx -4.).reshape((-1,1))
        y_vect= (np.arange(ny)*8./ny -4.).reshape((-1,1))

        data[:,:] = np.cos(np.sqrt((x_vect+2)**2 @ y_vect.T**2))
        self.graph.setData(x_vect, y_vect, data)

        plot.legend.setVisible(True)

        plot.colorScale = QCPColorScale(plot)
        plot.plotLayout().addElement(0, 1, plot.colorScale); # add it to the right of the main axis rect
        plot.colorScale.setType(QCPAxis.atRight) # scale shall be vertical bar with tick/axis labels right (actually atRight is already the default)

        self.graph.colorMap().setColorScale(plot.colorScale) # associate the color map with the color scale
        plot.colorScale.axis().setLabel("Magnetic Field Strength");

        # set the color gradient of the color map to one of the presets:
        self.graph.colorMap().setGradient(QCPColorGradient.GradientPreset.gpPolar)
        # we could have also created a QCPColorGradient instance and added own colors to
        # the gradient, see the documentation of QCPColorGradient for what's possible.

        # rescale the data dimension (color) such that all data points lie in the span visualized by the color gradient:
        self.graph.colorMap().rescaleDataRange();

        # make sure the axis rect and color scale synchronize their bottom and top margins (so they line up):
        plot.marginGroup = QCPMarginGroup(plot)
        plot.axisRect().setMarginGroup(QCP.msBottom|QCP.msTop, plot.marginGroup)
        plot.colorScale.setMarginGroup(QCP.msBottom|QCP.msTop, plot.marginGroup)

        # rescale the key (x) and value (y) axes so the whole color map is visible:
        plot.rescaleAxes()

        plot.setOpenGl(True)


if __name__ == '__main__':
    app = QApplication(sys.argv)
    w = MainWindow()
    w.show()
    app.exec()



#!/bin/env python3
# taken from https://github.com/radareorg/cutter/blob/master/src/bindings/src_list.py

import os
import sys
import xml.etree.ElementTree as Et
import argparse

parser = argparse.ArgumentParser(description='PySide/shiboken generated file list predictor')
parser.add_argument('--build-system', default='meson')
parser.add_argument('--bindings')
args = parser.parse_args()

bindings = args.bindings
build_system = args.build_system


def fix_name(name):
    return name.replace('<','_').replace('>','_')

def find_all_objects(node, found=None):
    found = found or []
    found += list(map(lambda t:fix_name(t.attrib['name']), filter(lambda n: n.get('generate','yes') == 'yes' , node.findall('object-type')+node.findall('value-type')+node.findall('interface-type'))))
    smart_ptrs = node.findall('smart-pointer-type')
    for sp in smart_ptrs:
        classes = sp.attrib.get('instantiations', '').split(',')
        for c in classes:
            found.append(f"{sp.attrib['name']}_{c}")
    for item in node.findall('namespace-type'):
        if item.attrib.get('visible', 'true') in ['true', 'auto']:
            found += [item.attrib['name']]
        found += list(map(lambda name:f"{item.attrib['name']}_{name}",find_all_objects(item)))
    return found


def get_cpp_files_gen(include_package=True):
    with open(bindings,'r') as f:
        xml=Et.fromstring(f.read())
        types = find_all_objects(xml)
        package = xml.attrib['package']

    cpp_files_gen = [f"{package.lower()}_module_wrapper.cpp"]+[f"{typename.lower()}_wrapper.cpp" for typename in types]

    if include_package:
        cpp_files_gen = [os.path.join(package, f) for f in cpp_files_gen]

    return cpp_files_gen


def cmd_cmake():
    sys.stdout.write(";".join(get_cpp_files_gen()))


def cmd_qmake():
    sys.stdout.write("\n".join(get_cpp_files_gen()) + "\n")


def cmd_meson():
    sys.stdout.write(";".join(get_cpp_files_gen(include_package=False)))


cmds = {"cmake": cmd_cmake, "qmake": cmd_qmake, "meson": cmd_meson}

cmds[build_system]()



#!/bin/env python3

import importlib
import argparse
import subprocess
import site
import os
import platform
from tempfile import TemporaryDirectory
from glob import glob

parser = argparse.ArgumentParser(description='PySide/shiboken wrapper rpath fix')
parser.add_argument('--wheel-file')
parser.add_argument('--pyside_version', default='2')
args = parser.parse_args()

pyside_ver = args.pyside_version

shiboken = importlib.import_module(f'shiboken{pyside_ver}')
PySide = importlib.import_module(f'PySide{pyside_ver}')


site_packages_path = site.getsitepackages()[0]

PySide_mod_rpath = '$ORIGIN/'+os.path.relpath(PySide.__path__[0], site_packages_path)
bundled_qt_rpath = f"{PySide_mod_rpath}/Qt/lib"
shiboken_mod_rpath = '$ORIGIN/'+os.path.relpath(shiboken.__path__[0], site_packages_path)

def fix_rpath_macos(extracted_wheel_folder):
    lib_file=glob(f'{extracted_wheel_folder}/SciQLopPlotsBindings.*.so')[0]
    subprocess.run(
        ['install_name_tool', '-add_rpath', '@loader_path/PySide6/Qt/lib/', lib_file],
        check=True
    )
    subprocess.run(
        ['install_name_tool', '-add_rpath', '@loader_path/PySide6/', lib_file],
        check=True
    )
    subprocess.run(
        ['install_name_tool', '-add_rpath', '@loader_path/shiboken6/', lib_file],
        check=True
    )


def fix_rpath_linux(extracted_wheel_folder):
    so_file=glob(f'{extracted_wheel_folder}/SciQLopPlotsBindings.*.so')[0]
    subprocess.run(
        ['patchelf', '--remove-rpath', so_file],
        check=True
    )

    subprocess.run(
        ['patchelf', '--set-rpath', ":".join([PySide_mod_rpath, bundled_qt_rpath, shiboken_mod_rpath]), so_file],
        check=True
    )


with TemporaryDirectory() as tmpdir:
    subprocess.run(
        ['wheel', 'unpack','-d', tmpdir,  args.wheel_file],
        check=True
    )
    extracted_wheel_folder = glob(f'{tmpdir}/sciqlopplots-*')[0]
    if platform.system().lower() == 'darwin':
        fix_rpath_macos(extracted_wheel_folder)
    else:
        fix_rpath_linux(extracted_wheel_folder)
    subprocess.run(
        ['wheel', 'pack','-d', 'dist', glob(f'{tmpdir}/sciqlopplots-*')[0]],
        check=True
    )



#!/bin/env python3

import os
import platform
import importlib
import importlib.machinery
import argparse
from glob import glob, escape
import re
import platform

parser = argparse.ArgumentParser(description='PySide/shiboken ')
group = parser.add_mutually_exclusive_group()
group.add_argument('--libs', action='store_true')
group.add_argument('--includes',action='store_true')
group.add_argument('--typesystem', action='store_true')
parser.add_argument('--modules')
parser.add_argument('--qmake')
parser.add_argument('--build-dir', nargs='?')
parser.add_argument('--pyside_version', default='2')
args = parser.parse_args()

pyside_ver = args.pyside_version

shiboken_generator = importlib.import_module(f'shiboken{pyside_ver}_generator')
shiboken = importlib.import_module(f'shiboken{pyside_ver}')
PySide = importlib.import_module(f'PySide{pyside_ver}')

PySide_mod_path = PySide.__path__[0]
shiboken_generator_mod_path = shiboken_generator.__path__[0]
shiboken_mod_path = shiboken.__path__[0]

build_dir = args.build_dir

if platform.system().lower() == 'darwin':
    ext_sufix = re.escape('.dylib')
else:
    ext_sufix = f"[{'|'.join(map(re.escape, importlib.machinery.EXTENSION_SUFFIXES))}]"


def first_existing_path(path_list):
    for path in path_list:
        if path is not None and os.path.exists(path):
            return path

def find_lib(name, search_folders):
    name_regex = re.compile(name)
    for folder in search_folders:
        if os.path.exists(folder):
            files=os.listdir(folder)
            found = list(filter(name_regex.match, files))
            if len(found):
                return f'{folder}/{found[0]}'

def make_link_flag(lib_path):
    basename = os.path.basename(lib_path)
    if basename.startswith('lib'):
        return f"-l{basename[3:].split('.so')[0]}"
    return lib_path


def make_link_flags(libs_paths):
    if platform.system().lower() == 'linux':
        link_flag="-Wl,-rpath="
    else:
        link_flag="-L"
    folders = list(set(list(map(lambda l: f"{link_flag}{os.path.relpath(os.path.dirname(l), build_dir)}",libs_paths))))
    #libs = list(map(make_link_flag, libs_paths))
    libs = libs_paths
    return folders + libs


if shiboken.__file__ and shiboken_generator.__file__ and PySide.__file__:
    PySide_inc = first_existing_path([f'{PySide_mod_path}/include',f'/usr/include/PySide{pyside_ver}'])
    PySide_typesys = first_existing_path([f'{PySide_mod_path}/typesystems','/usr/share/PySide{pyside_ver}/typesystems'])
    shiboken_includes = first_existing_path([f'{shiboken_mod_path}/include',f'{shiboken_generator_mod_path}/include',f'/usr/include/shiboken{pyside_ver}'])
    
    if args.typesystem:
        print(PySide_typesys)

    modules = args.modules.split(',')

    if args.libs:
        main_lib = [find_lib(f'libshiboken{pyside_ver}.*{ext_sufix}', [f'{shiboken_mod_path}', '/usr/lib64/'])]
        main_lib += [find_lib(f'lib[Pp]y[sS]ide{pyside_ver}\..*{ext_sufix}.*', [f'{PySide_mod_path}', '/usr/lib64/'])]
        if platform.system().lower() == 'darwin':
            modules_libs = []
        else:
            modules_libs = [importlib.import_module(f'PySide{pyside_ver}.{module}').__file__ for module in modules]
        print(" ".join(make_link_flags(main_lib + modules_libs)))

    if args.includes:
        modules_incs = [f"-I{PySide_inc}/{module}" for module in modules]
        print(" ".join([f"-I{PySide_inc} -I{shiboken_includes}"]+ modules_incs))
    
    



from SciQLopPlots import QCustomPlot, QCP, QCPColorMap, QCPRange, QCPColorScale, QCPAxis, QCPColorGradient, QCPMarginGroup
from PySide6.QtWidgets import QMainWindow, QApplication
from PySide6.QtGui import QPen, QColorConstants, QColor, QBrush
import sys
import math
import numpy as np
from types import SimpleNamespace


class MainWindow(QMainWindow):
    def __init__(self):
        QMainWindow.__init__(self)
        self._setup_ui()
        self.setGeometry(400, 250, 542, 390);

    def _setup_ui(self):
        plot: QCustomPlot = QCustomPlot(self)
        plot.setInteractions(QCP.iRangeDrag|QCP.iRangeZoom|QCP.iSelectPlottables)
        self.setCentralWidget(plot)
        self.graph = plot.addSciQLopGraph(plot.xAxis, plot.yAxis, ["X","Y","Z"])
        x=np.arange(3e7)*1.
        y=np.ones((3,len(x)))
        y[0]=np.cos(x/60)
        y[1]=np.cos(x/600)*1.3
        y[2]=np.cos(x/6000)*1.7
        self.graph.setData(x,y)

        self.graph.graphAt(0).setPen(QPen(QColorConstants.Red))
        self.graph.graphAt(1).setPen(QPen(QColorConstants.Blue))
        self.graph.graphAt(2).setPen(QPen(QColorConstants.Green))

        plot.legend.setVisible(True)

        plot.rescaleAxes()
        plot.setOpenGl(True)


if __name__ == '__main__':
    app = QApplication(sys.argv)
    w = MainWindow()
    w.show()
    app.exec()
